@@ -13,7 +13,7 @@ is preferred for real-time applications.
 
 """
 from skimage import data
-from skimage import transform as tf
+from skimage import transform
 from skimage.feature import (match_descriptors, corner_harris,
                              corner_peaks, ORB, plot_matches)
 from skimage.color import rgb2gray
@@ -21,10 +21,10 @@ import matplotlib.pyplot as plt
 
 
 img1 = rgb2gray(data.astronaut())
-img2 = tf.rotate(img1, 180)
-tform = tf.AffineTransform(scale=(1.3, 1.1), rotation=0.5,
-                           translation=(0, -200))
-img3 = tf.warp(img1, tform)
+img2 = transform.rotate(img1, 180)
+tform = transform.AffineTransform(scale=(1.3, 1.1), rotation=0.5,
+                                  translation=(0, -200))
+img3 = transform.warp(img1, tform)
 
 descriptor_extractor = ORB(n_keypoints=200)
 
