@@ -9,10 +9,15 @@ Thresholding is used to create a binary image from a grayscale image [1]_.
 
 .. seealso::
     A more comprehensive presentation on
-    :ref:`sphx_glr_auto_examples_xx_applications_plot_thresholding.py`
+    :ref:`sphx_glr_auto_examples_applications_plot_thresholding.py`
 
 """
 
+
+import matplotlib.pyplot as plt
+from skimage import data
+from skimage.filters import threshold_otsu
+
 ######################################################################
 # We illustrate how to apply one of these thresholding algorithms.
 # Otsu's method [2]_ calculates an "optimal" threshold (marked by a red line in the
@@ -20,14 +25,9 @@ Thresholding is used to create a binary image from a grayscale image [1]_.
 # which are separated by the threshold. Equivalently, this threshold minimizes
 # the intra-class variance.
 #
-# .. [2] http://en.wikipedia.org/wiki/Otsu's_method
+# .. [2] https://en.wikipedia.org/wiki/Otsu's_method
 #
 
-import matplotlib.pyplot as plt
-from skimage import data
-from skimage.filters import threshold_otsu
-
-
 image = data.camera()
 thresh = threshold_otsu(image)
 binary = image > thresh
@@ -58,7 +58,7 @@ plt.show()
 # underlying assumptions, it is often difficult to know which algorithm will give
 # the best results. Therefore, Scikit-image includes a function to evaluate
 # thresholding algorithms provided by the library. At a glance, you can select
-# the best algorithm for you data without a deep understanding of their
+# the best algorithm for your data without a deep understanding of their
 # mechanisms.
 #
 
