@@ -5,21 +5,23 @@ For more images, see
  - http://sipi.usc.edu/database/database.php
 
 """
-
-import os as _os
-
-import numpy as _np
+import sys
 from warnings import warn
+import numpy as np
+import shutil
 
 from ..util.dtype import img_as_bool
 from ._binary_blobs import binary_blobs
-from ._detect import lbp_frontal_face_cascade_filename
+from ._registry import registry, legacy_registry, registry_urls
+
+from .. import __version__
 
 import os.path as osp
-data_dir = osp.abspath(osp.dirname(__file__))
+import os
 
 __all__ = ['data_dir',
            'load',
+           'download_all',
            'astronaut',
            'binary_blobs',
            'brick',
@@ -50,6 +52,272 @@ __all__ = ['data_dir',
            'stereo_motorcycle']
 
 
+
+legacy_data_dir = osp.abspath(osp.dirname(__file__))
+skimage_distribution_dir = osp.join(legacy_data_dir, '..')
+
+try:
+    from pooch.utils import file_hash
+except ModuleNotFoundError:
+    # Function taken from
+    # https://github.com/fatiando/pooch/blob/master/pooch/utils.py
+    def file_hash(fname, alg="sha256"):
+        """
+        Calculate the hash of a given file.
+        Useful for checking if a file has changed or been corrupted.
+        Parameters
+        ----------
+        fname : str
+            The name of the file.
+        alg : str
+            The type of the hashing algorithm
+        Returns
+        -------
+        hash : str
+            The hash of the file.
+        Examples
+        --------
+        >>> fname = "test-file-for-hash.txt"
+        >>> with open(fname, "w") as f:
+        ...     __ = f.write("content of the file")
+        >>> print(file_hash(fname))
+        0fc74468e6a9a829f103d069aeb2bb4f8646bad58bf146bb0e3379b759ec4a00
+        >>> import os
+        >>> os.remove(fname)
+        """
+        import hashlib
+        if alg not in hashlib.algorithms_available:
+            raise ValueError(
+                "Algorithm '{}' not available in hashlib".format(alg))
+        # Calculate the hash in chunks to avoid overloading the memory
+        chunksize = 65536
+        hasher = hashlib.new(alg)
+        with open(fname, "rb") as fin:
+            buff = fin.read(chunksize)
+            while buff:
+                hasher.update(buff)
+                buff = fin.read(chunksize)
+        return hasher.hexdigest()
+
+
+def _has_hash(path, expected_hash):
+    """Check if the provided path has the expected hash."""
+    if not osp.exists(path):
+        return False
+    return file_hash(path) == expected_hash
+
+
+def create_image_fetcher():
+    try:
+        import pooch
+    except ImportError:
+        # Without pooch, fallback on the standard data directory
+        # which for now, includes a few limited data samples
+        return None, legacy_data_dir
+
+    # Pooch expects a `+` to exist in development versions.
+    # Since scikit-image doesn't follow that convention, we have to manually
+    # remove `.dev` with a `+` if it exists.
+    # This helps pooch understand that it should look in master
+    # to find the required files
+    pooch_version = __version__.replace('.dev', '+')
+    url = "https://github.com/scikit-image/scikit-image/raw/v0.17.x/skimage/"
+
+    # Create a new friend to manage your sample data storage
+    image_fetcher = pooch.create(
+        # Pooch uses appdirs to select an appropriate directory for the cache
+        # on each platform.
+        # https://github.com/ActiveState/appdirs
+        # On linux this converges to
+        # '$HOME/.cache/scikit-image'
+        # With a version qualifier
+        path=pooch.os_cache("scikit-image"),
+        base_url=url,
+        version=pooch_version,
+        env="SKIMAGE_DATADIR",
+        registry=registry,
+        urls=registry_urls,
+    )
+
+    data_dir = osp.join(str(image_fetcher.abspath), 'data')
+
+
+    return image_fetcher, data_dir
+
+
+image_fetcher, data_dir = create_image_fetcher()
+
+if image_fetcher is None:
+    has_pooch = False
+else:
+    has_pooch = True
+
+def _fetch(data_filename):
+    """Fetch a given data file from either the local cache or the repository.
+
+    This function provides the path location of the data file given
+    its name in the scikit-image repository.
+
+    Parameters
+    ----------
+    data_filename:
+        Name of the file in the scikit-image repository. e.g.
+        'restoration/tess/camera_rl.npy'.
+
+    Returns
+    -------
+    Path of the local file as a python string.
+
+    Raises
+    ------
+    KeyError:
+        If the filename is not known to the scikit-image distribution.
+
+    ModuleNotFoundError:
+        If the filename is known to the scikit-image distribution but pooch
+        is not installed.
+
+    ConnectionError:
+        If scikit-image is unable to connect to the internet but the
+        dataset has not been downloaded yet.
+    """
+    resolved_path = osp.join(data_dir, '..', data_filename)
+    expected_hash = registry[data_filename]
+
+    # Case 1:
+    # The file may already be in the data_dir.
+    # We may have decided to ship it in the scikit-image distribution.
+    if _has_hash(resolved_path, expected_hash):
+        # Nothing to be done, file is where it is expected to be
+        return resolved_path
+
+    # Case 2:
+    # The user is using a cloned version of the github repo, which
+    # contains both the publicly shipped data, and test data.
+    # In this case, the file would be located relative to the
+    # skimage_distribution_dir
+    gh_repository_path = osp.join(skimage_distribution_dir, data_filename)
+    if _has_hash(gh_repository_path, expected_hash):
+        parent = osp.dirname(resolved_path)
+        os.makedirs(parent, exist_ok=True)
+        shutil.copy2(gh_repository_path, resolved_path)
+        return resolved_path
+
+    # Case 3:
+    # Pooch not found.
+    if image_fetcher is None:
+        raise ModuleNotFoundError(
+            "The requested file is part of the scikit-image distribution, "
+            "but requires the installation of an optional dependency, pooch. "
+            "To install pooch, use your preferred python package manager. "
+            "Follow installation instruction found at "
+            "https://scikit-image.org/docs/stable/install.html"
+        )
+
+    # Case 4:
+    # Pooch needs to download the data. Let the image fetcher to search for
+    # our data. A ConnectionError is raised if no internet connection is
+    # available.
+    try:
+        resolved_path = image_fetcher.fetch(data_filename)
+    except ConnectionError as err:
+        # If we decide in the future to suppress the underlying 'requests'
+        # error, change this to `raise ... from None`. See PEP 3134.
+        raise ConnectionError(
+            'Tried to download a scikit-image dataset, but no internet '
+            'connection is available. To avoid this message in the '
+            'future, try `skimage.data.download_all()` when you are '
+            'connected to the internet.'
+        ) from err
+    return resolved_path
+
+
+def _init_pooch():
+    os.makedirs(data_dir, exist_ok=True)
+    shutil.copy2(osp.join(skimage_distribution_dir, 'data', 'README.txt'),
+                 osp.join(data_dir, 'README.txt'))
+
+    data_base_dir = osp.join(data_dir, '..')
+    # Fetch all legacy data so that it is available by default
+    for filename in legacy_registry:
+        _fetch(filename)
+
+
+# This function creates directories, and has been the source of issues for
+# downstream users, see
+# https://github.com/scikit-image/scikit-image/issues/4660
+# https://github.com/scikit-image/scikit-image/issues/4664
+if has_pooch:
+    _init_pooch()
+
+
+def download_all(directory=None):
+    """Download all datasets for use with scikit-image offline.
+
+    Scikit-image datasets are no longer shipped with the library by default.
+    This allows us to use higher quality datasets, while keeping the
+    library download size small.
+
+    This function requires the installation of an optional dependency, pooch,
+    to download the full dataset. Follow installation instruction found at
+
+        https://scikit-image.org/docs/stable/install.html
+
+    Call this function to download all sample images making them available
+    offline on your machine.
+
+    Parameters
+    ----------
+    directory: path-like, optional
+        The directory where the dataset should be stored.
+
+    Raises
+    ------
+    ModuleNotFoundError:
+        If pooch is not install, this error will be raised.
+
+    Notes
+    -----
+    scikit-image will only search for images stored in the default directory.
+    Only specify the directory if you wish to download the images to your own
+    folder for a particular reason. You can access the location of the default
+    data directory by inspecting the variable `skimage.data.data_dir`.
+    """
+
+    if image_fetcher is None:
+        raise ModuleNotFoundError(
+            "To download all package data, scikit-image needs an optional "
+            "dependency, pooch."
+            "To install pooch, follow our installation instructions found at "
+            "https://scikit-image.org/docs/stable/install.html"
+        )
+    # Consider moving this kind of logic to Pooch
+    old_dir = image_fetcher.path
+    try:
+        if directory is not None:
+            image_fetcher.path = directory
+
+        for filename in image_fetcher.registry:
+            _fetch(filename)
+    finally:
+        image_fetcher.path = old_dir
+
+
+def lbp_frontal_face_cascade_filename():
+    """Return the path to the XML file containing the weak classifier cascade.
+
+    These classifiers were trained using LBP features. The file is part
+    of the OpenCV repository [1]_.
+
+    References
+    ----------
+    .. [1] OpenCV lbpcascade trained files
+           https://github.com/opencv/opencv/tree/master/data/lbpcascades
+    """
+
+    return _fetch('data/lbpcascade_frontalface_opencv.xml')
+
+
 def load(f, as_gray=False):
     """Load an image file located in the data directory.
 
@@ -93,7 +361,7 @@ def _load(f, as_gray=False):
     # importing io is quite slow since it scans all the backends
     # we lazy import it here
     from ..io import imread
-    return imread(_os.path.join(data_dir, f), plugin='pil', as_gray=as_gray)
+    return imread(_fetch(f), as_gray=as_gray)
 
 
 def camera():
@@ -106,7 +374,7 @@ def camera():
     camera : (512, 512) uint8 ndarray
         Camera image.
     """
-    return _load("camera.png")
+    return _load("data/camera.png")
 
 
 def astronaut():
@@ -128,7 +396,7 @@ def astronaut():
         Astronaut image.
     """
 
-    return _load("astronaut.png")
+    return _load("data/astronaut.png")
 
 
 def brick():
@@ -136,7 +404,7 @@ def brick():
 
     Returns
     -------
-    brick: (512, 512) uint8 image
+    brick : (512, 512) uint8 image
         A small section of a brick wall.
 
     Notes
@@ -194,7 +462,7 @@ def brick():
     >>> brick = rotate(brick, -90)
     >>> imwrite('brick.png', img_as_ubyte(rgb2gray(brick)))
     """
-    return _load("brick.png", as_gray=True)
+    return _load("data/brick.png", as_gray=True)
 
 
 def grass():
@@ -202,7 +470,7 @@ def grass():
 
     Returns
     -------
-    grass: (512, 512) uint8 image
+    grass : (512, 512) uint8 image
         Some grass.
 
     Notes
@@ -242,23 +510,7 @@ def grass():
     >>> grass = skimage.img_as_ubyte(skimage.color.rgb2gray(grass_orig[:512, :512]))
     >>> imageio.imwrite('grass.png', grass)
     """
-    return _load("grass.png", as_gray=True)
-
-
-def rough_wall():
-    """Rough wall.
-
-    Returns
-    -------
-    rough_wall: (512, 512) uint8 image
-        Some rough wall.
-
-    """
-    from warnings import warn
-    warn("The rough_wall dataset has been removed due to licensing concerns."
-         "It has been replaced with the gravel dataset. This warning message"
-         "will be replaced with an error in scikit-image 0.17.", stacklevel=2)
-    return gravel()
+    return _load("data/grass.png", as_gray=True)
 
 
 def gravel():
@@ -266,7 +518,7 @@ def gravel():
 
     Returns
     -------
-    gravel: (512, 512) uint8 image
+    gravel : (512, 512) uint8 image
         Grayscale gravel sample.
 
     Notes
@@ -313,7 +565,7 @@ def gravel():
     >>> gravel = skimage.img_as_ubyte(skimage.color.rgb2gray(gravel[:512, :512]))
     >>> imageio.imwrite('gravel.png', gravel)
     """
-    return _load("gravel.png", as_gray=True)
+    return _load("data/gravel.png", as_gray=True)
 
 
 def text():
@@ -332,7 +584,7 @@ def text():
         Text image.
     """
 
-    return _load("text.png")
+    return _load("data/text.png")
 
 
 def checkerboard():
@@ -347,7 +599,7 @@ def checkerboard():
     checkerboard : (200, 200) uint8 ndarray
         Checkerboard image.
     """
-    return _load("chessboard_GRAY.png")
+    return _load("data/chessboard_GRAY.png")
 
 
 def cell():
@@ -378,7 +630,7 @@ def cell():
            for spherical objects in quantitative phase imaging." Optics Express
            26(8): 10729-10743 (2018). :DOI:`10.1364/OE.26.010729`
     """
-    return _load('cell.png')
+    return _load('data/cell.png')
 
 
 def coins():
@@ -403,7 +655,7 @@ def coins():
     coins : (303, 384) uint8 ndarray
         Coins image.
     """
-    return _load("coins.png")
+    return _load("data/coins.png")
 
 
 def logo():
@@ -414,7 +666,7 @@ def logo():
     logo : (500, 500, 4) uint8 ndarray
         Logo image.
     """
-    return _load("logo.png")
+    return _load("data/logo.png")
 
 
 def microaneurysms():
@@ -442,7 +694,7 @@ def microaneurysms():
            2013.
            :DOI:`10.1155/2013/154860`
     """
-    return _load("microaneurysms.png")
+    return _load("data/microaneurysms.png")
 
 
 def moon():
@@ -456,7 +708,7 @@ def moon():
     moon : (512, 512) uint8 ndarray
         Moon image.
     """
-    return _load("moon.png")
+    return _load("data/moon.png")
 
 
 def page():
@@ -470,7 +722,7 @@ def page():
     page : (191, 384) uint8 ndarray
         Page image.
     """
-    return _load("page.png")
+    return _load("data/page.png")
 
 
 def horse():
@@ -486,7 +738,7 @@ def horse():
     horse : (328, 400) bool ndarray
         Horse image.
     """
-    return img_as_bool(_load("horse.png", as_gray=True))
+    return img_as_bool(_load("data/horse.png", as_gray=True))
 
 
 def clock():
@@ -503,7 +755,7 @@ def clock():
     clock : (300, 400) uint8 ndarray
         Clock image.
     """
-    return _load("clock_motion.png")
+    return _load("data/clock_motion.png")
 
 
 def immunohistochemistry():
@@ -523,7 +775,7 @@ def immunohistochemistry():
     immunohistochemistry : (512, 512, 3) uint8 ndarray
         Immunohistochemistry image.
     """
-    return _load("ihc.png")
+    return _load("data/ihc.png")
 
 
 def chelsea():
@@ -541,7 +793,7 @@ def chelsea():
     chelsea : (300, 451, 3) uint8 ndarray
         Chelsea image.
     """
-    return _load("chelsea.png")
+    return _load("data/chelsea.png")
 
 
 def coffee():
@@ -560,7 +812,7 @@ def coffee():
     coffee : (400, 600, 3) uint8 ndarray
         Coffee image.
     """
-    return _load("coffee.png")
+    return _load("data/coffee.png")
 
 
 def hubble_deep_field():
@@ -584,7 +836,7 @@ def hubble_deep_field():
     hubble_deep_field : (872, 1000, 3) uint8 ndarray
         Hubble deep field image.
     """
-    return _load("hubble_deep_field.jpg")
+    return _load("data/hubble_deep_field.jpg")
 
 
 def retina():
@@ -611,7 +863,7 @@ def retina():
     retina : (1411, 1411, 3) uint8 ndarray
         Retina image in RGB.
     """
-    return _load("retina.jpg")
+    return _load("data/retina.jpg")
 
 
 def shepp_logan_phantom():
@@ -625,10 +877,10 @@ def shepp_logan_phantom():
 
     Returns
     -------
-    phantom: (400, 400) float64 image
+    phantom : (400, 400) float64 image
         Image of the Shepp-Logan phantom in grayscale.
     """
-    return _load("phantom.png", as_gray=True)
+    return _load("data/phantom.png", as_gray=True)
 
 
 def colorwheel():
@@ -636,10 +888,10 @@ def colorwheel():
 
     Returns
     -------
-    colorwheel: (370, 371, 3) uint8 image
+    colorwheel : (370, 371, 3) uint8 image
         A colorwheel.
     """
-    return _load("color.png")
+    return _load("data/color.png")
 
 
 def rocket():
@@ -662,7 +914,7 @@ def rocket():
     rocket : (427, 640, 3) uint8 ndarray
         Rocket image.
     """
-    return _load("rocket.jpg")
+    return _load("data/rocket.jpg")
 
 
 def stereo_motorcycle():
@@ -719,9 +971,11 @@ def stereo_motorcycle():
     .. [2] http://vision.middlebury.edu/stereo/data/scenes2014/
 
     """
-    return (_load("motorcycle_left.png"),
-            _load("motorcycle_right.png"),
-            _np.load(_os.path.join(data_dir, "motorcycle_disp.npz"))["arr_0"])
+    filename = _fetch("data/motorcycle_disp.npz")
+    disp = np.load(filename)['arr_0']
+    return (_load("data/motorcycle_left.png"),
+            _load("data/motorcycle_right.png"),
+            disp)
 
 
 def lfw_subset():
@@ -753,4 +1007,4 @@ def lfw_subset():
     .. [2] http://vis-www.cs.umass.edu/lfw/
 
     """
-    return _np.load(_os.path.join(data_dir, 'lfw_subset.npy'))
+    return np.load(_fetch('data/lfw_subset.npy'))
